#
# Author:: Frederic Nowak (<frederic.nowak@alphard.io>)
# Cookbook:: alphard-chef-deploy
# Definition:: configure
#
# Copyright:: 2017, Hydra Technologies, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
default_installer = proc do |_configurations|
  raise 'you must specify a installer'
end

define :configure, organization: nil, installer: default_installer do
  # Retrieves the organization from the definition parameters
  # If the name is nil, the recipe will stop and no further operation will be done
  organization = params[:organization]

  if organization.nil?
    log 'no_organization' do
      message 'an organization must be provided to this definition!'
      level :error
    end
    return
  end

  log 'organization' do
    message "organization: #{organization}"
    level :info
  end

  # Retrieves the name from the definition parameters
  # If the name is nil, the recipe will stop and no further operation will be done
  name = params[:name]

  if name.nil?
    log 'no_name' do
      message 'a name must be provided to this definition!'
      level :error
    end
    return
  end

  log 'name' do
    message "name: #{name}"
    level :info
  end

  # Retrieves the environment
  environment = node.chef_environment

  if environment.nil?
    log 'no_environment' do
      message "#{environment} must not be nil!"
      level :error
    end
    return
  end

  log 'environment' do
    message "environment: #{environment}"
    level :info
  end

  # Detects the platform
  platform_os = case node['platform_family']
                when 'windows'
                  'windows'
                when 'mac_os_x'
                  'macosx'
                else
                  'linux'
                end
  platform_arch = 1.size == 4 ? '32' : '64'
  platform = "#{platform_os}#{platform_arch}"

  log 'platform' do
    message "platform: #{platform}"
    level :info
  end

  # Retrieves the configuration. The node must be present in the environment.
  # If the node is nil, the recipe will stop and no further operation will be done.
  def mash(key = nil, p = nil)
    n = p.nil? ? node : p
    m = key.nil? ? n : n[key]
    m.nil? ? Mash.new : m
  end

  environment_configuration = mash.merge(environment: environment,
                                         platform: platform)
  organization_configuration = mash(organization).merge(organization: organization,
                                                        environment: environment,
                                                        platform: platform)
  default_cookbook_configuration = mash(nil, node['alphard']['deploy'])
  global_configuration = mash(:global, organization_configuration)
  local_configuration = mash(name, organization_configuration)

  if local_configuration.nil?
    log 'no_local_configuration' do
      message "a local configuration must be provided to the '#{name}' definition!"
      level :error
    end
    return
  end

  def deep_merge(min, mout)
    Mash.from_hash(
      min.to_hash.merge(mout.to_hash) do |_key, v1, v2|
        if v1.is_a?(Hash) && v2.is_a?(Hash)
          deep_merge(v1, v2)
        else
          v2
        end
      end
    )
  end

  configuration = default_cookbook_configuration
  configuration = deep_merge(configuration, global_configuration)
  configuration = deep_merge(configuration, local_configuration)

  # Retrieves the packaging from the definition parameters
  # If the name is nil, the recipe will stop and no further operation will be done
  packaging = configuration[:packaging]

  if packaging.nil?
    log 'no_packaging' do
      message 'a packaging must be provided to this definition!'
      level :error
    end
    return
  end

  log 'packaging' do
    message "packaging: #{packaging}"
    level :info
  end

  # Retrieves the version. The version must be present in the environment.
  # If the version is nil, the recipe will stop and no further operation will be done.
  version = configuration[:version]

  if version.nil?
    log 'no_version' do
      message 'a version must be provided to this definition!'
      level :error
    end
    return
  end

  log 'version' do
    message "version: #{version}"
    level :info
  end

  is_snapshot = !version.to_s.index('-SNAPSHOT').nil?

  log 'snapshot' do
    message "snapshot: #{is_snapshot}"
    level :info
  end

  # Retrieves the description and set it to the empty string is is not present in the environment.
  description = configuration[:description]
  description = '' if description.nil?

  log 'description' do
    message "description: #{description}"
    level :info
  end

  # Retrieves the user. The version must be present in the environment.
  # If the user is nil, the recipe will stop and no further operation will be done.
  user = configuration[:user]

  if user.nil?
    log 'no_user' do
      message 'a user must be provided to this definition!'
      level :error
    end
    return
  end

  log 'user' do
    message "user: #{user}"
    level :info
  end

  group = configuration[:group]

  if group.nil?
    log 'no_group' do
      message 'a group must be provided to this definition!'
      level :error
    end
    return
  end

  log 'group' do
    message "group: #{group}"
    level :info
  end

  # Checks if the installation must be cleaned
  clean = (is_snapshot && configuration[:clean] != 'false') || configuration[:clean] == 'true'

  log 'clean' do
    message "clean: #{clean}"
    level :info
  end

  # Checks if the directory already exists.
  # If the directory does not exist, it creates it with the provided user and group.
  # Creates other directories
  directory = "#{configuration[:base]}/#{name}"

  directory directory do
    owner user
    group group
    mode '0755'
    recursive true
    action :create
  end

  # Retrieves the previous version. The version is located in a file.
  # If the file exists, the current is set with the content of the file.
  # If the file doesn't exist the current version is set to 'nil'
  version_file = "#{directory}/#{configuration[:version_file]}"

  previous_version = nil
  if File.exist?(version_file)
    file = File.open(version_file, 'rb')
    previous_version = file.read
  end

  log 'previous_version' do
    message "previous version: #{previous_version}"
    level :info
  end

  # Creates directories
  archive_directory = "#{directory}/archive"
  next_archive_directory = "#{archive_directory}/#{version}"
  version_directory_name = 'version'
  version_directory = "#{directory}/#{version_directory_name}"
  next_version_directory = "#{version_directory}/#{version}"
  current_directory = "#{directory}/current"

  if clean || version != previous_version
    [directory, archive_directory, next_archive_directory,
     version_directory, next_version_directory, current_directory].each do |d|
      directory d do
        owner user
        group group
        mode '0755'
        action :create
      end
    end
  end

  log 'previous_version' do
    message "previous version: #{previous_version}"
    level :info
  end

  log 'directories' do
    message <<EOF
directory: #{directory}
archive directory: #{next_archive_directory}
version directory: #{next_version_directory}
current directory: #{current_directory}
EOF
    level :info
  end

  # Updates configurations
  configuration = configuration.merge(name: name,
                                      is_snapshot: is_snapshot,
                                      clean: clean,
                                      previous_version: previous_version,
                                      organization: organization,
                                      environment: environment,
                                      platform: platform,
                                      base_directory: directory,
                                      archive_directory: next_archive_directory,
                                      version_directory: next_version_directory,
                                      current_directory: current_directory)
  configurations = { environment_configuration: environment_configuration,
                     organization_configuration: organization_configuration,
                     configuration: configuration }

  log 'json_configuration' do
    message JSON.pretty_generate(JSON.parse(configuration.to_json))
    level :debug
  end

  # Triggers the installer
  installer = params[:installer]
  installer.call(configurations)

  if clean || version != previous_version
    # Deletes old versions except current and previous version ones
    find_command = "find #{version_directory}/ * -maxdepth 0 -type d"
    find_command += " ! -name '#{version_directory_name}' ! -name '#{version}'"
    find_command += " ! -name '#{previous_version}' -exec rm -rf '{}' ';'"
    execute find_command do
      command find_command
      returns [0, 1]
      user user
      group group
      cwd version_directory
      action :run
    end

    # Deletes old archives except current and previous version ones
    find_command = "find #{archive_directory}/ * -maxdepth 0 -type f"
    find_command += " ! -name '*#{version}*' ! -name '*#{previous_version}*'"
    find_command += " -exec rm -f '{}' ';'"
    execute find_command do
      command find_command
      returns [0, 1]
      user user
      group group
      cwd archive_directory
      action :run
    end

    # Writes the version in the version file
    file version_file.to_s do
      owner user
      group group
      content version
      action :create
    end
  end
end
